@@ -13,7 +13,17 @@ import pydantic
 
 from uptrain.framework.checks import CheckSet, ExperimentArgs
 from uptrain.framework.base import Settings
-from uptrain.framework.evals import Evals, ParametricEval, CritiqueTone, GuidelineAdherence, ResponseMatching
+from uptrain.framework.evals import (
+    Evals,
+    JailbreakDetection,
+    ParametricEval,
+    CritiqueTone,
+    GuidelineAdherence,
+    ResponseMatching,
+    ConversationSatisfaction,
+)
+from uptrain.framework.rca_templates import RcaTemplate
+from uptrain.utilities import polars_to_pandas
 
 
 class DataSchema(pydantic.BaseModel):
@@ -23,6 +33,21 @@ class DataSchema(pydantic.BaseModel):
     context: str = "context"
     ground_truth: str = "ground_truth"
 
+    # Used for conversation evals
+    conversation: str = "conversation"
+
+    # Used for RcaTemplate: RAG_WITH_CITATION
+    cited_context: str = "cited_context"
+
+    # TODO: Remove
+    scenario: str = "scenario"
+    objective: str = "objective"
+
+    ##rag_evals
+    sub_questions: str = "sub_questions"
+    reranked_context: str = "reranked_context"
+    concise_context: str = "concise_context"
+
 
 def raise_or_return(response: httpx.Response):
     if not response.is_success:
@@ -43,28 +68,23 @@ class APIClientWithoutAuth:
         server_url = settings.check_and_get("uptrain_server_url")
         self.base_url = server_url.rstrip("/") + "/api/open"
         self.client = httpx.Client(
-            timeout=httpx.Timeout(50, connect=5),
+            timeout=httpx.Timeout(7200, connect=5),
         )
 
     def evaluate(
         self,
         data: list[dict],
-        checks: list[t.Union[Evals, ParametricEval]],
+        checks: list[t.Union[Evals, ParametricEval, dict]],
         metadata: dict,
     ):
-        """Run an evaluation on the UpTrain server (Doesn't require UpTrain API Key).
-        """
+        """Run an evaluation on the UpTrain server (Doesn't require UpTrain API Key)."""
 
         url = f"{self.base_url}/evaluate_no_auth"
         response_json = []
         try:
             response = self.client.post(
                 url,
-                json={
-                    "data": data,
-                    "checks": checks,
-                    "metadata": metadata
-                },
+                json={"data": data, "checks": checks, "metadata": metadata},
             )
             response_json = raise_or_return(response)
         except Exception as e:
@@ -74,7 +94,6 @@ class APIClientWithoutAuth:
         return response_json
 
 
-
 class APIClient:
     base_url: str
     client: httpx.Client
@@ -92,7 +111,7 @@ class APIClient:
         self.base_url = server_url.rstrip("/") + "/api/public"
         self.client = httpx.Client(
             headers={"uptrain-access-token": api_key},
-            timeout=httpx.Timeout(50, connect=5),
+            timeout=httpx.Timeout(7200, connect=5),
         )
 
     def check_auth(self):
@@ -225,9 +244,30 @@ class APIClient:
         response = self.client.get(url)
         return raise_or_return(response)
 
-    def download_run_result(self, run_id: str, check_name: str, fpath: str) -> None:
+    def get_run_results(self, run_id: str, check_name: str) -> list[dict]:
         """Get the results of a run.
 
+        Args:
+            run_id: unique identifier for the run.
+            check_name: name of the check to get results for.
+
+        Returns:
+            run: information about the run along with a unique identifier.
+        """
+        url = f"{self.base_url}/run/{run_id}/results"
+        params: dict = {"check_name": check_name}
+        response = self.client.get(url, params=params)
+        if not response.is_success:
+            logger.error(response.text)
+            response.raise_for_status()
+        else:
+            return pl.DataFrame(
+                polars_to_pandas(pl.read_ndjson(response.content))
+            ).to_dicts()
+
+    def download_run_result(self, run_id: str, check_name: str, fpath: str) -> None:
+        """Download the results of a run.
+
         Args:
             run_id: unique identifier for the run.
             check_name: name of the check to get results for.
@@ -256,7 +296,14 @@ class APIClient:
         response = self.client.get(url, params=params)
         return raise_or_return(response)
 
-    def add_daily_schedule(self, checkset: str, start_on: str, assign_topics: int = 0, assign_topics_args: t.Optional[dict] = None):
+    def add_daily_schedule(
+        self,
+        checkset: str,
+        start_on: str,
+        assign_topics: int = 0,
+        assign_topics_args: t.Optional[dict] = None,
+        extra_args: t.Optional[dict] = None,
+    ):
         """Schedules a periodic evaluation on the server. Specify the checkset to run against it.
 
         Args:
@@ -268,13 +315,14 @@ class APIClient:
         """
         url = f"{self.base_url}/schedule"
         response = self.client.post(
-            url, 
+            url,
             json={
                 "checkset": checkset,
                 "start_on": start_on,
                 "assign_topics": assign_topics,
-                "assign_topics_args": assign_topics_args
-            }
+                "assign_topics_args": assign_topics_args,
+                "extra_args": extra_args,
+            },
         )
         return raise_or_return(response)
 
@@ -333,6 +381,27 @@ class APIClient:
         response = self.client.put(url, params=params)
         return raise_or_return(response)
 
+    def get_schedule_results_all_dates(
+        self, schedule_id: str, check_name: str
+    ) -> list[dict]:
+        """Get the results of a schedule.
+
+        Args:
+            schedule_id: unique identifier for the schedule.
+            check_name: name of the check to get results for.
+
+        Returns:
+            run: information about the schedule along with a unique identifier.
+        """
+        url = f"{self.base_url}/schedule/{schedule_id}/results"
+        params: dict = {"check_name": check_name}
+        response = self.client.get(url, params=params)
+        if not response.is_success:
+            logger.error(response.text)
+            response.raise_for_status()
+        else:
+            return pl.read_ndjson(response.content).to_dicts()
+
     def evaluate(
         self,
         eval_name: str,
@@ -349,22 +418,29 @@ class APIClient:
         elif isinstance(full_dataset, pd.DataFrame):
             full_dataset = full_dataset.to_dict(orient="records")
 
-        # send in chunks of 50, so the connection doesn't time out waiting for the server
+        # send in chunks of 100, so the connection doesn't time out waiting for the server
         results = []
+
+        if params is not None:
+            params["uptrain_settings"] = self.settings.dict()
+        else:
+            params = {}
+            params["uptrain_settings"] = self.settings.dict()
+
         NUM_TRIES = 3
-        for i in range(0, len(full_dataset), 10):
+        for i in range(0, len(full_dataset), 100):
             response_json = None
             for try_num in range(NUM_TRIES):
                 try:
                     logger.info(
-                        f"Sending evaluation request for rows {i} to <{i+10} to the Uptrain server"
+                        f"Sending evaluation request for rows {i} to <{i+100} to the Uptrain server"
                     )
                     response = self.client.post(
                         url,
                         json={
                             "eval_name": eval_name,
-                            "dataset": full_dataset[i : i + 10],
-                            "params": params if params is not None else {},
+                            "dataset": full_dataset[i : i + 100],
+                            "params": params,
                         },
                     )
                     response_json = raise_or_return(response)
@@ -380,11 +456,103 @@ class APIClient:
 
         return results
 
+    def perform_root_cause_analysis(
+        self,
+        project_name: str,
+        data: t.Union[list[dict], pl.DataFrame, pd.DataFrame],
+        rca_template: RcaTemplate,
+        scenario_description: t.Optional[str] = None,
+        schema: t.Union[DataSchema, dict[str, str], None] = None,
+        metadata: t.Optional[dict[str, t.Any]] = None,
+    ):
+        """Perform root cause analysis on the server.
+
+        Args:
+            project_name: Name of the project to run root cause analysis on.
+            data: Data to do rca on. Either a Pandas DataFrame or a list of dicts.
+            rca_template: rca template to run.
+            schema: Schema of the data. Only required if the data attributes aren't typical (question, response, context).
+            metadata: Attributes to attach to this dataset. Useful for filtering and grouping in the UI.
+
+        Returns:
+            results: List of dictionaries with each data point and corresponding error modes.
+        """
+
+        url = f"{self.base_url}/perform_root_cause_analysis"
+
+        if isinstance(data, pl.DataFrame):
+            data = data.to_dicts()
+        elif isinstance(data, pd.DataFrame):
+            data = data.to_dict(orient="records")
+
+        if schema is None:
+            schema = DataSchema()
+        elif isinstance(schema, dict):
+            schema = DataSchema(**schema)
+
+        if metadata is None:
+            metadata = {}
+        metadata["internal_call"] = metadata.get("internal_call", False)
+
+        req_attrs, ser_templates = set(), []
+        if rca_template == RcaTemplate.RAG_WITH_CITATION:
+            req_attrs.update(
+                [schema.question, schema.response, schema.context, schema.cited_context]
+            )
+        else:
+            raise Exception("RCA Template not supported yet")
+
+        dictn = {"scenario_description": scenario_description}
+        ser_templates.append({"rca_template_name": rca_template.value, **dictn})
+
+        for idx, row in enumerate(data):
+            if not req_attrs.issubset(row.keys()):
+                raise ValueError(
+                    f"Row {idx} is missing required all required attributes for evaluation: {req_attrs}"
+                )
+
+        # send in chunks of 50, so the connection doesn't time out waiting for the server
+        results = []
+        NUM_TRIES, BATCH_SIZE = 3, 50
+        for i in range(0, len(data), BATCH_SIZE):
+            response_json = None
+            for try_num in range(NUM_TRIES):
+                try:
+                    logger.info(
+                        f"Sending root cause analysis request for rows {i} to <{i+BATCH_SIZE} to the Uptrain server"
+                    )
+                    response = self.client.post(
+                        url,
+                        json={
+                            "data": data[i : i + BATCH_SIZE],
+                            "rca_templates": ser_templates,
+                            "metadata": {
+                                "project": project_name,
+                                "schema": schema.dict(),
+                                **metadata,
+                                "uptrain_settings": self.settings.dict(),
+                            },
+                        },
+                    )
+                    response_json = raise_or_return(response)
+                    break
+                except Exception as e:
+                    logger.info("Retrying root cause analysis request")
+                    if try_num == NUM_TRIES - 1:
+                        logger.error(f"Evaluation failed with error: {e}")
+                        raise e
+
+            if response_json is not None:
+                results.extend(response_json)
+
+        return results
+
     def log_and_evaluate(
         self,
         project_name: str,
         data: t.Union[list[dict], pl.DataFrame, pd.DataFrame],
         checks: list[t.Union[str, Evals, ParametricEval]],
+        scenario_description: t.Union[str, list[str], None] = None,
         schema: t.Union[DataSchema, dict[str, str], None] = None,
         metadata: t.Optional[dict[str, t.Any]] = None,
     ):
@@ -422,21 +590,67 @@ class APIClient:
 
         req_attrs, ser_checks = set(), []
         for m in checks:
-            if m in [Evals.FACTUAL_ACCURACY, Evals.RESPONSE_COMPLETENESS_WRT_CONTEXT, Evals.RESPONSE_CONSISTENCY]:
+            if m in [Evals.SUB_QUERY_COMPLETENESS]:
+                req_attrs.update([schema.sub_questions, schema.question])
+            elif m in [Evals.CONTEXT_CONCISENESS]:
+                req_attrs.update(
+                    [schema.question, schema.context, schema.concise_context]
+                )
+            elif m in [Evals.CONTEXT_RERANKING]:
+                req_attrs.update(
+                    [schema.question, schema.context, schema.reranked_context]
+                )
+            elif m in [
+                Evals.FACTUAL_ACCURACY,
+                Evals.RESPONSE_COMPLETENESS_WRT_CONTEXT,
+                Evals.RESPONSE_CONSISTENCY,
+                Evals.CODE_HALLUCINATION,
+            ]:
                 req_attrs.update([schema.question, schema.context, schema.response])
-            elif m in [Evals.RESPONSE_RELEVANCE, Evals.RESPONSE_COMPLETENESS, Evals.RESPONSE_CONCISENESS]:
+            elif m in [
+                Evals.VALID_RESPONSE,
+                Evals.RESPONSE_RELEVANCE,
+                Evals.RESPONSE_COMPLETENESS,
+                Evals.RESPONSE_CONCISENESS,
+                Evals.PROMPT_INJECTION,
+            ]:
                 req_attrs.update([schema.question, schema.response])
             elif m in [Evals.CONTEXT_RELEVANCE]:
                 req_attrs.update([schema.question, schema.context])
-            elif m == Evals.CRITIQUE_LANGUAGE or isinstance(m, CritiqueTone) or isinstance(m, GuidelineAdherence):
+            elif (
+                m in [Evals.CRITIQUE_LANGUAGE]
+                or isinstance(m, CritiqueTone)
+                or isinstance(m, GuidelineAdherence)
+            ):
                 req_attrs.update([schema.response])
+            elif m in [
+                Evals.RESPONSE_ALIGNMENT_WITH_SCENARIO,
+                Evals.RESPONSE_SINCERITY_WITH_SCENARIO,
+            ]:
+                req_attrs.update(
+                    [
+                        schema.question,
+                        schema.response,
+                        schema.scenario,
+                        schema.objective,
+                    ]
+                )
             elif isinstance(m, ResponseMatching):
-                req_attrs.update([schema.response, schema.ground_truth])
+                req_attrs.update(
+                    [schema.question, schema.response, schema.ground_truth]
+                )
+            elif isinstance(m, ConversationSatisfaction):
+                req_attrs.update([schema.conversation])
+            elif isinstance(m, JailbreakDetection):
+                req_attrs.update([schema.question])
 
             if isinstance(m, ParametricEval):
-                ser_checks.append({"check_name": m.__class__.__name__, **m.dict()})
+                dictm = m.dict()
+                dictm.update({"scenario_description": scenario_description})
+                ser_checks.append({"check_name": m.__class__.__name__, **dictm})
             elif isinstance(m, Evals):
-                ser_checks.append(m.value)
+                dictm = {"scenario_description": scenario_description}
+                ser_checks.append({"check_name": m.value, **dictm})
             else:
                 raise ValueError(f"Invalid metric: {m}")
         for idx, row in enumerate(data):
@@ -481,7 +695,6 @@ class APIClient:
 
         return results
 
-
     def evaluate_experiments(
         self,
         project_name: str,
@@ -507,7 +720,7 @@ class APIClient:
         if metadata is None:
             metadata = {}
 
-        metadata.update({'uptrain_experiment_columns': exp_columns})
+        metadata.update({"uptrain_experiment_columns": exp_columns})
 
         if schema is None:
             schema = DataSchema()
@@ -525,11 +738,13 @@ class APIClient:
         results = pl.DataFrame(results)
         all_cols = set(results.columns)
         value_cols = list(all_cols - set([schema.question] + exp_columns))
-        exp_results = results.pivot(values=value_cols, index=schema.question, columns=exp_columns)
+        index_cols = metadata.get("uptrain_index_columns", [schema.question])
+        exp_results = results.pivot(
+            values=value_cols, index=index_cols, columns=exp_columns
+        )
         exp_results = exp_results.to_dicts()
         return exp_results
 
-
     def download_project_eval_results(self, project_name: str, fpath: str):
         """Fetch all the evaluation results for a project.
 
